unit frmMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  waveio, MMSystem, ExtCtrls, StdCtrls, Spin;

const
  BUFFER_LENGTH = 100;
  NUM_BUFFERS = 8;
  GRAPH_WIDTH = 200;

type
  TFormMain = class(TForm)
    PaintBox1: TPaintBox;
    Button1: TButton;
    Button2: TButton;
    OpenDialog1: TOpenDialog;
    procedure PaintBox1Paint(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    ws: TFileWaveStream;
    PCMReader: TPCMWaveReader;
    Headers: array [0..NUM_BUFFERS-1] of TWaveHdr;
    BufferLength: Longint;
    CurrentBuffer, Ending: Integer;
    waveout: HWaveOut;
    Graphic: array [0..GRAPH_WIDTH-1, 0..1] of Byte;
    Closing: Boolean;

    procedure OpenFile( FileName: String );
    procedure CloseFile;
    procedure WriteBuffer;
    procedure CalcGraphic;
    procedure Stop( var Msg: TMessage ); message WM_USER;
    procedure Done( var Msg: TMessage ); message MM_WOM_DONE;
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

{$R *.DFM}

procedure TFormMain.FormCreate(Sender: TObject);
var
  Cnt: Integer;
begin
  for Cnt := 0 to GRAPH_WIDTH-1 do
  begin
    Graphic[Cnt, 0] := 0;
    Graphic[Cnt, 1] := 0;
  end;
end;


procedure TFormMain.Button1Click(Sender: TObject);
begin
  CloseFile;

  Close;
end;

procedure TFormMain.Button2Click(Sender: TObject);
begin
  if OpenDialog1.Execute then
  begin
    CloseFile;
    OpenFile( OpenDialog1.FileName );
  end;
end;


procedure TFormMain.OpenFile( FileName: String );
var
  Cnt: Integer;
begin
  ws := TFileWaveStream.Create( FileName, nil );
  PCMReader := TPCMWaveReader.Create( ws );

  CurrentBuffer := 0;
  Ending := 1;
  Closing := False;
  BufferLength := ((PCMReader.Format^.nAvgBytesPerSec * BUFFER_LENGTH)
    div (1000 * PCMReader.Format^.nBlockAlign)) * PCMReader.Format^.nBlockAlign;

  if (PCMReader.Size > 0) then
  begin
    waveOutOpen( @waveout, WAVE_MAPPER, PCMReader.Format, Handle,
      Integer(self), CALLBACK_WINDOW or WAVE_ALLOWSYNC );

    for Cnt := 0 to NUM_BUFFERS-1 do
    begin
      ZeroMemory(@Headers[Cnt], sizeof(Headers[Cnt]));
      with Headers[Cnt] do
      begin
        GetMem( lpData, BufferLength );
        dwBufferLength := BufferLength;
        dwFlags := WHDR_DONE;
      end;
      waveOutPrepareHeader( waveout, @(Headers[Cnt]), sizeof(Headers[Cnt]) );
    end;

    for Cnt := 0 to NUM_BUFFERS-1 do
      WriteBuffer;

    CalcGraphic;
  end;
end;

procedure TFormMain.CloseFile;
var
  Cnt: Integer;
  p: Pointer;
begin
  if (waveout <> 0) then
  begin
    Closing := True;

    waveOutReset( waveout );

    for Cnt := 0 to NUM_BUFFERS-1 do
    begin
      p := Headers[Cnt].lpData;
      waveOutUnprepareHeader( waveout, @Headers[Cnt], sizeof(Headers[Cnt]) );
      FreeMem( p );
    end;

    waveOutClose( waveout );

    waveout := 0;
  end;

  if (PCMReader <> nil) then
    PCMReader.Free;
  PCMReader := nil;

  if (ws <> nil) then
    ws.Free;
  ws := nil;
end;


procedure TFormMain.WriteBuffer;
var
  sw: Integer;
begin
  if (Closing) then exit;

  sw := PCMReader.Read( Headers[CurrentBuffer].lpData^,
    BufferLength div PCMReader.Format^.nBlockAlign )
    * PCMReader.Format^.nBlockAlign;
  if (sw > 0) then
  begin
    if BufferLength <> sw then
      if PCMReader.Format^.wBitsPerSample = 8 then
        FillMemory( PChar(Headers[CurrentBuffer].lpData) + sw,
          BufferLength - sw, 128 )
      else
        ZeroMemory( PChar(Headers[CurrentBuffer].lpData) + sw,
          BufferLength - sw );
    waveOutWrite( waveout, @Headers[CurrentBuffer],
      sizeof(Headers[CurrentBuffer]) );
  end
  else
    if (Ending < NUM_BUFFERS) then
      Inc( Ending )
    else
      PostMessage( Handle, WM_USER, 0, 0 );

  CurrentBuffer := (CurrentBuffer + 1) mod NUM_BUFFERS;
end;


procedure TFormMain.PaintBox1Paint(Sender: TObject);
var
  Cnt: Integer;
begin
  for Cnt := 0 to GRAPH_WIDTH-1 do
  begin
    PaintBox1.Canvas.MoveTo( Cnt, (255-Graphic[Cnt, 0]) shr 1 );
    PaintBox1.Canvas.LineTo( Cnt, (255-Graphic[Cnt, 1]) shr 1 );
  end;
end;

procedure TFormMain.CalcGraphic;
var
  Cnt, Cnt2, NumSamps: Integer;
  upper, lower, AddOn, v: Byte;
  ps: PByte;
begin
  if (Closing) or (Ending = NUM_BUFFERS) then exit;

  NumSamps := BufferLength div (PCMReader.Format^.nBlockAlign * GRAPH_WIDTH);
  ps := PByte(Headers[(CurrentBuffer + 1) mod NUM_BUFFERS].lpData);
  if (PCMReader.Format^.wBitsPerSample = 16) then
  begin
    Inc(ps);
    AddOn := 128;
  end
  else
    AddOn := 0;

  for Cnt := 0 to GRAPH_WIDTH-1 do
  begin
    upper := 0;
    lower := 255;
    for Cnt2 := 0 to NumSamps-1 do
    begin
      v := ps^+AddOn;
      if v > upper then upper := v;
      if v < lower then lower := v;
      Inc( ps, PCMReader.Format^.nBlockAlign );
    end;

    Graphic[Cnt, 0] := lower;
    Graphic[Cnt, 1] := upper;
  end;
end;

procedure TFormMain.Stop( var Msg: TMessage );
begin
  CloseFile;
end;

procedure TFormMain.Done( var Msg: TMessage );
begin
  WriteBuffer;
  CalcGraphic;
  PaintBox1.Invalidate;
end;


end.
